SystemOrganization addCategory: #'Obj1-preguntados'!
SystemOrganization addCategory: 'Obj1-preguntados-model'!
SystemOrganization addCategory: 'Obj1-preguntados-specs'!

TestCase subclass: #TestCategoria
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-specs'!

!TestCategoria methodsFor: 'as yet unclassified' stamp: 'FedericoAloi 11/18/2015 16:40'!
test_SiguientePreguntaFallaSiNoHayMasPreguntas
	|categoria pregunta|
	pregunta := Pregunta new.
	categoria := Categoria conPreguntas: { pregunta }.
	
	categoria siguientePregunta.

	self should: [categoria siguientePregunta] raise: Error whoseDescriptionIncludes: 'No quedan más preguntas en esta categoría' description: ''! !

!TestCategoria methodsFor: 'as yet unclassified' stamp: 'FedericoAloi 11/18/2015 12:30'!
test_SiguientePreguntaQuitaLaPreguntaDeLaCategoria
	|categoria pregunta1 pregunta2 siguiente|
	pregunta1 := Pregunta new.
	pregunta2 := Pregunta new.
	categoria := Categoria conPreguntas: { pregunta1. pregunta2 }.
	
	siguiente := categoria siguientePregunta.
	
	self deny: (categoria preguntas includes: siguiente)! !

TestCase subclass: #TestPreguntados
	instanceVariableNames: 'jugadorDesafiante otroJugador partida preguntaHistoria respuestaCorrecta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-specs'!

!TestPreguntados methodsFor: 'helpers' stamp: 'FedericoAloi 11/24/2015 19:46'!
contestarBienPreguntaActual	
	| turno pregunta |
	turno := partida turnoActual.
	pregunta := turno pregunta.
	turno responderPreguntaCon: turno pregunta respuestaCorrecta.
	
	^ pregunta! !

!TestPreguntados methodsFor: 'helpers' stamp: 'FedericoAloi 11/19/2015 16:57'!
contestarMalPreguntaActual
	| turno |
	turno := partida turnoActual.
	turno avanzar.
	turno responderPreguntaCon: (preguntaHistoria algunaRespuestaIncorrecta).
	
	^turno
	! !

!TestPreguntados methodsFor: 'helpers' stamp: 'FedericoAloi 11/24/2015 19:44'!
inicializarYContestarBienPreguntaActual
	| turno |
	turno := partida turnoActual.
	turno avanzar.
	
	^ self contestarBienPreguntaActual! !

!TestPreguntados methodsFor: 'running' stamp: 'FedericoAloi 11/24/2015 19:37'!
setUp
	jugadorDesafiante := Jugador llamado: 'Fede'.
	otroJugador := Jugador llamado: 'Ari'.
	
	preguntaHistoria  := Pregunta 
		texto: '¿En qué año fue proscripto el Peronismo?'
		respuestaCorrecta: '1955'
		eIncorrecta: '1954'
		eIncorrecta: '1956'
		eIncorrecta: '1953'.
		
	Categoria conPreguntasDeHistoria: { 
		preguntaHistoria. 
		Pregunta 
			texto: 'De las siguientes personas, ¿quién nunca ganó el Premio Nobel de la Paz?'
			respuestaCorrecta: 'Carlitos Balá'
			eIncorrecta: 'Barack Obama'
			eIncorrecta: 'Rigoberta Menchú'
			eIncorrecta: 'Adolfo Pérez Esquivel'. 
		Pregunta 
			texto: '¿En qué ciudad japonesa cayó la primera bomba atómica, arrojada durante la Segunda Guerra Mundial?'
			respuestaCorrecta: 'Hiroshima'
			eIncorrecta: 'Nagasaki'
			eIncorrecta: 'Tokio'
			eIncorrecta: 'Yokohama'. 
		Pregunta 
			texto: '¿Qué presidente de la República Argentina terminó su mandato tras abandonar la Casa Rosada en helicóptero?'
			respuestaCorrecta: 'Fernando de la Rúa'
			eIncorrecta: 'Hector Cámpora'
			eIncorrecta: 'Néstor Kirchner'
			eIncorrecta: 'Eduardo Duhalde'
	}.
	
	partida := Partida con: jugadorDesafiante desafiaA: otroJugador yCategorias: Categoria todas! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/18/2015 10:45'!
test_AlAvanzarEnUnTurnoEnEtapaGiroSeEstableceLaPregunta
	partida turnoActual avanzar.	
	self assert: partida turnoActual pregunta equals: preguntaHistoria! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/18/2015 10:15'!
test_AlComenzarUnTurnoElMismoSeEncuentraEnLaEtapaDeGiroDeRuleta
	self assert: partida turnoActual etapa equals: Etapa giroDeRuleta! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/24/2015 19:43'!
test_AlContestarBienLaPreguntaSeEligeOtraPregunta
	| preguntaRespondidaCorrectamente |
	preguntaRespondidaCorrectamente := self inicializarYContestarBienPreguntaActual.
	
	self deny: (partida turnoActual pregunta = preguntaRespondidaCorrectamente)! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/24/2015 19:43'!
test_AlContestarBienLaPreguntaSeSumaUnPuntoAlTurno
	self inicializarYContestarBienPreguntaActual.
	
	self assert: partida turnoActual puntaje equals: 1! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/19/2015 16:57'!
test_AlContestarMalLaPreguntaCambiaElTurno
	| turnoAnterior |
	turnoAnterior := self contestarMalPreguntaActual.	
		
	self deny: (turnoAnterior = partida turnoActual)! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/19/2015 16:57'!
test_AlContestarMalLaPreguntaElNuevoTurnoEsDelOtroJugador
	| turnoAnterior |
	turnoAnterior := self contestarMalPreguntaActual.	
		
	self assert: (partida turnoActual jugadorActual) equals: (turnoAnterior jugadorSiguiente)! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/19/2015 16:57'!
test_AlContestarMalLaPreguntaNoSumaPuntosAlTurno
	| turnoAnterior |
	turnoAnterior := self contestarMalPreguntaActual.	
		
	self assert: turnoAnterior puntaje equals: 0.! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/18/2015 10:14'!
test_AlIniciarseUnaPartidaElPrimerTurnoCorrespondeAlJugadorDesafiante
	self assert: partida turnoActual jugador equals: jugadorDesafiante! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/19/2015 13:48'!
test_AlIniciarseUnaPartidaNoHayPreguntaActual
	self should: [partida turnoActual pregunta] raise: Error whoseDescriptionIncludes: 'Aún no se ha sorteado la pregunta' description: ''! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/24/2015 21:23'!
test_TrasContestar3PreguntasCorrectamenteSeAvanzaAEtapaDeCorona
	self inicializarYContestarBienPreguntaActual.
	2 timesRepeat: [ self contestarBienPreguntaActual ].
	
	self assert: partida turnoActual etapa equals: Etapa obtencionDeCorona
	! !

!TestPreguntados methodsFor: 'tests' stamp: 'FedericoAloi 11/18/2015 10:14'!
test_UnJugadorPuedeIniciarUnaPartidaDesafiandoAOtroJugador
	self assert: partida estaIniciada

! !

Object subclass: #Categoria
	instanceVariableNames: 'preguntas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

Categoria class
	instanceVariableNames: 'historia'!

Categoria class
	instanceVariableNames: 'historia'!

!Categoria class methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 11:57'!
conPreguntas: unasPreguntas 
	^(self new) conPreguntas: unasPreguntas ; yourself! !

!Categoria class methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 11:57'!
conPreguntasDeHistoria: preguntasHistoria 
	historia := self conPreguntas: preguntasHistoria! !

!Categoria class methodsFor: 'public' stamp: 'FedericoAloi 11/18/2015 11:58'!
todas
	^{historia}! !

!Categoria methodsFor: 'initialization' stamp: 'FedericoAloi 11/19/2015 17:09'!
conPreguntas: unasPreguntas
	preguntas := OrderedCollection new.
	preguntas addAll: unasPreguntas! !

!Categoria methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 12:24'!
preguntas
	^ preguntas! !

!Categoria methodsFor: 'public' stamp: 'FedericoAloi 11/19/2015 17:09'!
siguientePregunta
	|siguiente|
	self validarQuedanPreguntas.	
	siguiente := preguntas first.
	preguntas remove: siguiente.
	^siguiente! !

!Categoria methodsFor: 'private' stamp: 'FedericoAloi 11/18/2015 16:39'!
validarQuedanPreguntas
	self preguntas ifEmpty: [self error: 'No quedan más preguntas en esta categoría']! !

Object subclass: #Etapa
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!Etapa commentStamp: '<historical>' prior: 0!
Soy una etapa de juego. Denoto el estado del juego en un momento dado.!

!Etapa class methodsFor: 'accesing' stamp: 'FedericoAloi 11/24/2015 21:26'!
giroDeRuleta
	^EtapaGiroDeRuleta ! !

!Etapa class methodsFor: 'accesing' stamp: 'FedericoAloi 11/24/2015 21:26'!
obtencionDeCorona
	^EtapaObtencionDeCorona ! !

!Etapa class methodsFor: 'initialization' stamp: 'FedericoAloi 11/24/2015 21:30'!
reiniciar! !

Etapa subclass: #EtapaGiroDeRuleta
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!EtapaGiroDeRuleta commentStamp: '<historical>' prior: 0!
Soy la etapa en la cual el jugador debe girar la ruleta!

!EtapaGiroDeRuleta class methodsFor: 'public' stamp: 'FedericoAloi 11/24/2015 21:27'!
avanzarEn: unTurno 
	(self etapaTerminadaPara: unTurno) 
		ifTrue: [ unTurno cambiarEtapaA: Etapa obtencionDeCorona ]
		ifFalse: [ unTurno pregunta: (self sortearPreguntaPara: unTurno) ]! !

!EtapaGiroDeRuleta class methodsFor: 'private' stamp: 'FedericoAloi 11/24/2015 21:28'!
etapaTerminadaPara: unTurno 
	^ unTurno puntaje = self puntajeMaximoEtapa! !

!EtapaGiroDeRuleta class methodsFor: 'constants' stamp: 'FedericoAloi 11/24/2015 19:55'!
puntajeMaximoEtapa
	^3! !

!EtapaGiroDeRuleta class methodsFor: 'private' stamp: 'FedericoAloi 11/24/2015 21:28'!
sortearCategoriaPara: unTurno 
	^unTurno categorias anyOne! !

!EtapaGiroDeRuleta class methodsFor: 'private' stamp: 'FedericoAloi 11/24/2015 21:29'!
sortearPreguntaPara: unTurno 
	^(self sortearCategoriaPara: unTurno) siguientePregunta! !

Etapa subclass: #EtapaObtencionDeCorona
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

Object subclass: #Jugador
	instanceVariableNames: 'nombre'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!Jugador commentStamp: '<historical>' prior: 0!
Soy un jugador del juego Preguntados!

!Jugador class methodsFor: 'as yet unclassified' stamp: 'FedericoAloi 11/19/2015 17:19'!
llamado: unNombre
	^(self new) conNombre: unNombre ; yourself! !

!Jugador methodsFor: 'initializers' stamp: 'FedericoAloi 11/19/2015 17:22'!
conNombre: unNombre
	nombre := unNombre! !

!Jugador methodsFor: 'public' stamp: 'ArielAlvarez 10/27/2015 21:24'!
desafiaA: otroJugador 
	^Partida newConDesafiante: self yDesafiado: otroJugador! !

Object subclass: #Partida
	instanceVariableNames: 'desafiante desafiado turno categorias'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!Partida commentStamp: '<historical>' prior: 0!
Soy una partida de preguntados!

!Partida class methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 12:01'!
con: desafiante desafiaA: contrincante yCategorias: categorias
	^(self new) inicializarConDesafiante: desafiante contra: contrincante yCategorias: categorias ; yourself! !

!Partida methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 12:09'!
categorias
	^ categorias! !

!Partida methodsFor: 'public' stamp: 'ArielAlvarez 10/27/2015 21:25'!
estaIniciada
	^true! !

!Partida methodsFor: 'initialization' stamp: 'FedericoAloi 11/19/2015 16:43'!
inicializarConDesafiante: unJugador contra: otroJugador yCategorias: unasCategorias
	desafiante := unJugador.
	desafiado := otroJugador.
	categorias := Bag new.
	categorias addAll: unasCategorias.
	self nuevoTurnoEntre: unJugador y: otroJugador.
	^self! !

!Partida methodsFor: 'public' stamp: 'ArielAlvarez 10/27/2015 21:40'!
jugador
	self shouldBeImplemented.! !

!Partida methodsFor: 'private' stamp: 'FedericoAloi 11/19/2015 16:44'!
nuevoTurnoEntre: unJugador y: otroJugador
	turno := (Turno new) inicializarConActual: unJugador ySiguiente: otroJugador enPartida: self! !

!Partida methodsFor: 'public' stamp: 'FedericoAloi 11/19/2015 16:46'!
siguienteTurno
	self nuevoTurnoEntre: (turno jugadorSiguiente) y: (turno jugadorActual)! !

!Partida methodsFor: 'public' stamp: 'AlvarezAriel 11/17/2015 22:36'!
turnoActual
	^turno
! !

Object subclass: #Pregunta
	instanceVariableNames: 'texto respuestas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!Pregunta class methodsFor: 'as yet unclassified' stamp: 'FedericoAloi 11/18/2015 16:48'!
texto: unTexto respuestaCorrecta: textoCorrecta eIncorrecta: textoIncorrecta1 eIncorrecta: textoIncorrecta2 eIncorrecta: textoIncorrecta3
	|pregunta|
	pregunta := (self new) conTexto: unTexto yRespuestas: 
		{ RespuestaCorrecta conTexto: textoCorrecta },
		({ textoCorrecta. textoIncorrecta1. textoIncorrecta2. textoIncorrecta3 } collect: [ :t | RespuestaIncorrecta conTexto: t ]).
		
	^pregunta! !

!Pregunta methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 16:58'!
algunaRespuestaIncorrecta
	^respuestas last! !

!Pregunta methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 16:50'!
conTexto: unTexto yRespuestas: unasRespuestas
	texto := unTexto.
	respuestas := OrderedCollection new.
	respuestas addAll: unasRespuestas ! !

!Pregunta methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 16:47'!
respuestaCorrecta
	^respuestas first! !

Object subclass: #Respuesta
	instanceVariableNames: 'texto'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!Respuesta class methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 11:00'!
conTexto: unTexto
	^(self new) conTexto: unTexto ; yourself! !

!Respuesta methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 11:00'!
conTexto: unTexto
	texto := unTexto ! !

Respuesta subclass: #RespuestaCorrecta
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!RespuestaCorrecta methodsFor: 'public' stamp: 'FedericoAloi 11/18/2015 16:55'!
fueElegidaEn: unTurno 
	unTurno preguntaRespondidaCorrectamente! !

Respuesta subclass: #RespuestaIncorrecta
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!RespuestaIncorrecta methodsFor: 'public' stamp: 'FedericoAloi 11/18/2015 16:59'!
fueElegidaEn: unTurno 
	unTurno preguntaRespondidaIncorrectamente ! !

Object subclass: #Turno
	instanceVariableNames: 'jugadorActual jugadorSiguiente etapa partida pregunta puntaje'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Obj1-preguntados-model'!

!Turno commentStamp: '<historical>' prior: 0!
Soy un turno en una partida. !

!Turno methodsFor: 'public' stamp: 'FedericoAloi 11/18/2015 12:04'!
avanzar
	etapa avanzarEn: self! !

!Turno methodsFor: 'public' stamp: 'FedericoAloi 11/24/2015 19:56'!
cambiarEtapaA: unaEtapa
	unaEtapa reiniciar.
	etapa := unaEtapa! !

!Turno methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 12:08'!
categorias
	^partida categorias! !

!Turno methodsFor: 'accesing' stamp: 'ArielAlvarez 10/28/2015 01:44'!
etapa
	^etapa! !

!Turno methodsFor: 'private' stamp: 'FedericoAloi 11/18/2015 16:55'!
incrementarPuntaje
	puntaje := puntaje + 1! !

!Turno methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 12:08'!
inicializarConActual: aJugador ySiguiente: aJugador2 enPartida: unaPartida
	jugadorActual := aJugador.
	jugadorSiguiente := aJugador2.
	partida := unaPartida.
	etapa := Etapa giroDeRuleta.
	^self! !

!Turno methodsFor: 'initialization' stamp: 'FedericoAloi 11/18/2015 16:56'!
initialize
	puntaje := 0! !

!Turno methodsFor: 'public' stamp: 'ArielAlvarez 10/27/2015 21:44'!
jugador
	^jugadorActual ! !

!Turno methodsFor: 'accessors' stamp: 'FedericoAloi 11/19/2015 16:47'!
jugadorActual
	^ jugadorActual! !

!Turno methodsFor: 'accessors' stamp: 'FedericoAloi 11/19/2015 16:46'!
jugadorSiguiente
	^ jugadorSiguiente! !

!Turno methodsFor: 'accessors' stamp: 'FedericoAloi 11/19/2015 13:49'!
pregunta
	^ pregunta ifNil: [ self error: 'Aún no se ha sorteado la pregunta' ]! !

!Turno methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 12:16'!
pregunta: unaPregunta 
	pregunta := unaPregunta! !

!Turno methodsFor: 'public' stamp: 'FedericoAloi 11/19/2015 17:01'!
preguntaRespondidaCorrectamente
	self incrementarPuntaje.
	self avanzar! !

!Turno methodsFor: 'public' stamp: 'FedericoAloi 11/19/2015 16:46'!
preguntaRespondidaIncorrectamente
	partida siguienteTurno
	! !

!Turno methodsFor: 'accessors' stamp: 'FedericoAloi 11/18/2015 16:56'!
puntaje
	^ puntaje! !

!Turno methodsFor: 'public' stamp: 'FedericoAloi 11/18/2015 16:54'!
responderPreguntaCon: unaRespuesta
	unaRespuesta fueElegidaEn: self! !
